;.......script written by trancexx (trancexx at yahoo dot com)

#AutoIt3Wrapper_Au3Check_Parameters=-d -w 1 -w 2 -w 3 -w 4 -w 5 -w 6
#include-once

; #INDEX# ===================================================================================
; Title ...............: OnHungApp
; File Name............: OnHungApp.au3
; Min. AutoIt Version..: v3.3.2.0
; Description .........: OnHungApp handler
; Author... ...........: trancexx
; Dll .................: kernel32.dll, user32.dll
; ===========================================================================================

; #CONSTANTS# ===============================================================================
Global Const $HUNGAPP_CONTINUEHANDLER = 259
Global Const $HUNGAPP_EXITHANDLER = 0
Global Const $HUNGAPP_EXITPROCESS = 1
; Pseudo-handle for DLL to use
Global Const $hHUNGAPPDLL__KERNEL32 = DllOpen("kernel32.dll")
;============================================================================================

; #CURRENT# =================================================================================
;_OnHungAppRegister
;_OnHungAppRegisterEx
;_OnHungAppUnRegister
; ===========================================================================================

; #FUNCTION# ;===============================================================================
; Name...........: _OnHungAppRegister
; Description ...: Registers function to call when specified window is not responding.
; Syntax.........: _OnHungAppRegister($hWnd, $sFunction[, $iInterval = Default[, $vParam1 = Default[, $vParam2 = Default[, $iExitCode = Default]]]])
; Parameters ....: $hWnd - A handle to the window to be monitored.
;                  $sFunction - Function to call when 'not responding' state is detected.
;                  $iInterval - Optional time interval in which to repeat the tesing. Default is 1000 ms (1sec).
;                  $vParam1 - Optional parameter to pass to the function (lparam). Default is 0.
;                  $vParam2 - Optional parameter to pass to the function (wparam). Default is 0.
;                  $iExitCode - Optional exit code for the app to return. Default is 0.
; Return values .: Success - Returns 1
;                  Failure - Returns 0 and sets @error:
;                  |1 - Windows function needed for the function are unavailable. Check @extended to tetermine which one.
;                  |2 - Couldn't get hanlde to the main thread.
;                  |3 - Couldn't allocate needed memory.
;                  |4 - Failed to register specified AutoIt function.
;                  |5 - Couldn't set PAGE_EXECUTE for the allocated memory.
;                  |6 - Failed to create thread for the handler.
; Author ........: trancexx
;============================================================================================
Func _OnHungAppRegister($hWnd, $sFunction, $iInterval = Default, $vParam1 = Default, $vParam2 = Default, $iExitCode = Default)
	; Deal with possible input
	If $iInterval = Default Or $iInterval = -1 Then $iInterval = 1000
	If $vParam1 = Default Then $vParam1 = 0
	If $vParam2 = Default Then $vParam2 = 0
	If $iExitCode = Default Then $iExitCode = 0
	; Get addresses of functions to use
	Local $pIsHungAppWindow = _HUNGAPP_GetAddress(_HUNGAPP_GetModuleHandle("user32.dll"), "IsHungAppWindow")
	If @error Then Return SetError(1, 1, 0)
	Local $hKernel32 = _HUNGAPP_GetModuleHandle("kernel32.dll")
	Local $pSleep = _HUNGAPP_GetAddress($hKernel32, "Sleep")
	If @error Then Return SetError(1, 2, 0)
	Local $pSuspendThread = _HUNGAPP_GetAddress($hKernel32, "SuspendThread")
	If @error Then Return SetError(1, 3, 0)
	Local $pResumeThread = _HUNGAPP_GetAddress($hKernel32, "ResumeThread")
	If @error Then Return SetError(1, 4, 0)
	Local $pSwitchToThread = _HUNGAPP_GetAddress($hKernel32, "SwitchToThread")
	If @error Then Return SetError(1, 5, 0)
	Local $pExitProcess = _HUNGAPP_GetAddress($hKernel32, "ExitProcess")
	If @error Then Return SetError(1, 6, 0)
	; Get handle of the main thread
	Local $hProcessPseudo = _HUNGAPP_GetCurrentProcess()
	Local $hThreadPseudo = _HUNGAPP_GetCurrentThread()
	; Pseude to real handle:
	Local $hMainThraed = _HUNGAPP_DuplicateHandle($hProcessPseudo, $hThreadPseudo, $hProcessPseudo, 0, 0, 2) ; DUPLICATE_SAME_ACCESS
	If @error Then Return SetError(2, 0, 0)
	; Allocate memory space
	Local $iCodeSize = 512 ; for example. It's more than enough
	Local $pCodeBuffer = _HUNGAPP_VirtualAlloc(0, $iCodeSize)
	If @error Then
		_HUNGAPP_CloseHandle($hMainThraed) ; close opened handle
		Return SetError(3, 0, 0)
	EndIf
	; AutoIt function to call from another thread
	Local $hFunction = DllCallbackRegister($sFunction, "int", "lparam;wparam")
	If @error Then
		_HUNGAPP_CloseHandle($hMainThraed) ; close opened handle
		Return SetError(4, 0, 0)
	EndIf
	; Structure in place of allocated space
	Local $tCodeBuffer = DllStructCreate("byte[" & $iCodeSize & "]", $pCodeBuffer)
	; Write code
	If @AutoItX64 Then
		DllStructSetData($tCodeBuffer, 1, _
				"0x" & _
				"4883EC" & _HUNGAPP_SwapEndian(40, 1) & _                            ; sub rsp, 40d ;<- expand the stack
				"" & _ ; 10. SLEEP FOR $iInterval
				"B9" & _HUNGAPP_SwapEndian($iInterval, 4) & _                        ; mov ecx, $iInterval
				"48B8" & _HUNGAPP_SwapEndian($pSleep, 8) & _                         ; mov rax, $pSleep
				"FFD0" & _                                                           ; call rax
				"" & _ ; 20. CHECK IF BLOCKED
				"48B9" & _HUNGAPP_SwapEndian($hWnd, 8) & _                           ; mov rcx,  $hWnd ;<- window handle
				"48B8" & _HUNGAPP_SwapEndian($pIsHungAppWindow, 8) & _               ; mov rax, $pIsHungAppWindow
				"FFD0" & _                                                           ; call rax
				"" & _ ; 30. SEE WHAT'S RETURNED
				"85C0" & _                                                           ; test eax, eax
				"" & _ ; 40. CONDITION - IF 0 RETURNED THEN...
				"74" & _HUNGAPP_SwapEndian(-43, 1) & _                               ; je -43d ;<- if IsHungAppWindow returned 0 jump back 43 bytes (to 10)
				"" & _ ; INCREASE SUSPEND COUNT FOR THE MAIN THREAD
				"48B9" & _HUNGAPP_SwapEndian($hMainThraed, 8) & _                    ; mov rcx, $hMainThraed
				"48B8" & _HUNGAPP_SwapEndian($pSuspendThread, 8) & _                 ; mov rax, $pSuspendThread
				"FFD0" & _                                                           ; call rax
				"" & _ ; 50. CALL AUTOIT FUNCTION
				"48BA" & _HUNGAPP_SwapEndian($vParam2, 8) & _                        ; mov rdx, $vParam2
				"48B9" & _HUNGAPP_SwapEndian($vParam1, 8) & _                        ; mov rcx, $vParam1
				"48B8" & _HUNGAPP_SwapEndian(DllCallbackGetPtr($hFunction), 8) & _   ; mov rax, $pFunction
				"FFD0" & _                                                           ; call rax
				"" & _ ; 60. SEE IF $HUNGAPP_CONTINUEHANDLER RETURNED
				"3D" & _HUNGAPP_SwapEndian(259, 4) & _                               ; cmp eax, 259d ;<- compare returned with 259
				"" & _ ; 70. CONDITION - IF NOT THEN...
				"75" & _HUNGAPP_SwapEndian(39, 1) & _                                ; jne 39d ;<- if not 259 returned jump forward 39 bytes
				"" & _ ; 80. IF YES THEN...
				"48B9" & _HUNGAPP_SwapEndian($hMainThraed, 8) & _                    ; mov rcx, $hMainThraed
				"48B8" & _HUNGAPP_SwapEndian($pResumeThread, 8) & _                  ; mov rax, $pResumeThread
				"FFD0" & _                                                           ; call rax
				"48B8" & _HUNGAPP_SwapEndian($pSwitchToThread, 8) & _                ; mov rax, $pSwitchToThread
				"FFD0" & _                                                           ; call rax
				"E9" & _HUNGAPP_SwapEndian(-143, 4) & _                              ; jmp -143d ; jump back 143 bytes
				"" & _ ; 90. SEE WHAT'S RETURNED AGAIN. THIS TIME TRUE/FALSE
				"85C0" & _                                                           ; test eax, eax
				"" & _ ; 100. CONDITION - IF FALSE THEN...
				"74" & _HUNGAPP_SwapEndian(17, 1) & _                                ; je 17d ;<- if 0 returned jump forward 17 bytes
				"" & _ ; 110. CONDITION - IF TRUE THEN...
				"B9" & _HUNGAPP_SwapEndian($iExitCode, 4) & _                        ; mov ecx, $iExitCode , param 1
				"48B8" & _HUNGAPP_SwapEndian($pExitProcess, 8) & _                   ; mov rax, $pExitProcess
				"FFD0" & _                                                           ; call rax
				"" & _ ; 120. RESUME MAIN THREAD
				"48B9" & _HUNGAPP_SwapEndian($hMainThraed, 8) & _                    ; mov rcx, $hMainThraed
				"48B8" & _HUNGAPP_SwapEndian($pResumeThread, 8) & _                  ; mov rax, $pResumeThread
				"FFD0" & _                                                           ; call rax
				"" & _ ; 130. SET RET VALUE TO 0
				"4831C0" & _                                                         ; xor rax, rax
				"4883C4" & _HUNGAPP_SwapEndian(40, 1) & _                            ; add rsp, 40d ;<- shrink stack
				"" & _ ; 140. RETURN
				"C3" _                                                               ; ret
				)
	Else
		DllStructSetData($tCodeBuffer, 1, _
				"0x" & _
				"" & _ ; 10. SLEEP FOR $iInterval
				"68" & _HUNGAPP_SwapEndian($iInterval, 4) & _                        ; push $iInterval
				"B8" & _HUNGAPP_SwapEndian($pSleep, 4) & _                           ; mov eax, $pSleep
				"FFD0" & _                                                           ; call eax
				"" & _ ; 20. CHECK IF BLOCKED
				"68" & _HUNGAPP_SwapEndian($hWnd, 4) & _                             ; push window handle
				"B8" & _HUNGAPP_SwapEndian($pIsHungAppWindow, 4) & _                 ; mov eax, $pIsHungAppWindow
				"FFD0" & _                                                           ; call eax
				"" & _ ; 30. SEE WHAT'S RETURNED
				"85C0" & _                                                           ; test eax, eax
				"" & _ ; 40. CONDITION - IF 0 RETURNED THEN...
				"74" & _HUNGAPP_SwapEndian(-28, 1) & _                               ; je -28d ;<- if IsHungAppWindow returned 0 jump back 28 bytes (to 10)
				"" & _ ; INCREASE SUSPEND COUNT FOR THE MAIN THREAD
				"68" & _HUNGAPP_SwapEndian($hMainThraed, 4) & _                      ; push $hMainThraed
				"B8" & _HUNGAPP_SwapEndian($pSuspendThread, 4) & _                   ; mov eax,  $pSuspendThread
				"FFD0" & _                                                           ; call eax
				"" & _ ; 50. CALL AUTOIT FUNCTION
				"68" & _HUNGAPP_SwapEndian($vParam2, 4) & _                          ; push $vParam2
				"68" & _HUNGAPP_SwapEndian($vParam1, 4) & _                          ; push $vParam1
				"B8" & _HUNGAPP_SwapEndian(DllCallbackGetPtr($hFunction), 4) & _     ; mov eax, $pFunction
				"FFD0" & _                                                           ; call eax
				"" & _ ; 60. SEE IF $HUNGAPP_CONTINUEHANDLER RETURNED
				"3D" & _HUNGAPP_SwapEndian(259, 4) & _                               ; cmp eax, 259d ;<- compare returned with 259
				"" & _ ; 70. CONDITION - IF NOT THEN...
				"75" & _HUNGAPP_SwapEndian(21, 1) & _                                ; jne 21d ;<- if not 259 returned jump forward 21 bytes
				"" & _ ; 80. IF YES THEN...
				"68" & _HUNGAPP_SwapEndian($hMainThraed, 4) & _                      ; push  $hMainThraed
				"B8" & _HUNGAPP_SwapEndian($pResumeThread, 4) & _                    ; mov eax, $pResumeThread
				"FFD0" & _                                                           ; call eax
				"B8" & _HUNGAPP_SwapEndian($pSwitchToThread, 4) & _                  ; mov eax, $pSwitchToThread
				"FFD0" & _                                                           ; call eax
				"EB" & _HUNGAPP_SwapEndian(-85, 1) & _                               ; jmp -85d ; jump back 85 bytes
				"" & _ ; 90. SEE WHAT'S RETURNED AGAIN. THIS TIME TRUE/FALSE
				"85C0" & _                                                           ; test eax, eax
				"" & _ ; 100. CONDITION - IF FALSE THEN...
				"74" & _HUNGAPP_SwapEndian(12, 1) & _                                ; je 12d ;<- if 0 returned jump forward 12 bytes
				"" & _ ; 110. CONDITION - IF TRUE THEN..
				"68" & _HUNGAPP_SwapEndian($iExitCode, 4) & _                        ; push $iExitCode
				"B8" & _HUNGAPP_SwapEndian($pExitProcess, 4) & _                     ; mov eax, $pExitProcess
				"FFD0" & _                                                           ; call eax
				"" & _ ; 120. RESUME MAIN THREAD
				"68" & _HUNGAPP_SwapEndian($hMainThraed, 4) & _                      ; push $hMainThraed
				"B8" & _HUNGAPP_SwapEndian($pResumeThread, 4) & _                    ; mov eax, $pResumeThread
				"FFD0" & _                                                           ; call eax
				"" & _ ; 130. SET RET VALUE TO 0
				"31C0" & _                                                           ; xor eax, eax
				"" & _ ; 140. RETURN
				"C3" _                                                               ; ret
				)
	EndIf
	; Set PAGE_EXECUTE for the code buffer
	_HUNGAPP_VirtualProtect($pCodeBuffer, $iCodeSize, 16) ; PAGE_EXECUTE
	If @error Then
		_HUNGAPP_CloseHandle($hMainThraed) ; close opened handle
		_HUNGAPP_VirtualFree($pCodeBuffer) ; free allocated memory
		Return SetError(5, 0, 0)
	EndIf
	; Create hread to run code in
	Local $hThread = _HUNGAPP_CreateThread($pCodeBuffer)
	If @error Then
		_HUNGAPP_CloseHandle($hMainThraed) ; close opened handle
		_HUNGAPP_VirtualFree($pCodeBuffer) ; free allocated memory
		Return SetError(6, 0, 0)
	EndIf
	; Arrange return
	Local $aOut[5] = [0x48756E67, $hMainThraed, $pCodeBuffer, $hThread, $hFunction] ; first element is magic num
	; Return sucess
	Return $aOut
EndFunc   ;==>_OnHungAppRegister

; #FUNCTION# ;===============================================================================
; Name...........: _OnHungAppRegisterEx
; Description ...: Registers function to call when specified window is not responding.
; Syntax.........: _OnHungAppRegisterEx($hWnd, $sFunction[, $iHungAppTime = Default[, $vParam1 = Default[, $vParam2 = Default[, $iExitCode = Default]]]])
; Parameters ....: $hWnd - A handle to the window to be monitored.
;                  $sFunction - Function to call when 'not responding' state is detected.
;                  $iHungAppTime - Time value in ms at which window will be flagged as unresponding one. Default is 5000 ms. Minimum is 500 ms.
;                  $vParam1 - Optional parameter to pass to the function (lparam). Default is 0.
;                  $vParam2 - Optional parameter to pass to the function (wparam). Default is 0.
;                  $iExitCode - Optional exit code for the app to return. Default is 0.
;                  $iInterval - Optional time interval in which to repeat the tesing. Default is 250 ms.
; Return values .: Success - Returns 1
;                  Failure - Returns 0 and sets @error:
;                  |1 - Windows function needed for the function are unavailable. Check @extended to tetermine which one.
;                  |2 - Couldn't get hanlde to the main thread.
;                  |3 - Couldn't allocate needed memory.
;                  |4 - Failed to register specified AutoIt function.
;                  |5 - Couldn't set PAGE_EXECUTE for the allocated memory.
;                  |6 - Failed to create thread for the handler.
; Author ........: trancexx
;============================================================================================
Func _OnHungAppRegisterEx($hWnd, $sFunction, $iHungAppTime = Default, $vParam1 = Default, $vParam2 = Default, $iExitCode = Default, $iInterval = Default)
	; Deal with possible input
	If $iHungAppTime = Default Or $iHungAppTime = -1 Then $iHungAppTime = 5000
	If $iHungAppTime < 500 Then $iHungAppTime = 500
	If $vParam1 = Default Then $vParam1 = 0
	If $vParam2 = Default Then $vParam2 = 0
	If $iExitCode = Default Then $iExitCode = 0
	If $iInterval = Default Or $iInterval = -1 Then $iInterval = 250
	; Get addresses of functions to use
	Local $pSendMessageTimeout = _HUNGAPP_GetAddress(_HUNGAPP_GetModuleHandle("user32.dll"), "SendMessageTimeoutW")
	If @error Then Return SetError(1, 1, 0)
	Local $hKernel32 = _HUNGAPP_GetModuleHandle("kernel32.dll")
	Local $pSleep = _HUNGAPP_GetAddress($hKernel32, "Sleep")
	If @error Then Return SetError(1, 2, 0)
	Local $pSuspendThread = _HUNGAPP_GetAddress($hKernel32, "SuspendThread")
	If @error Then Return SetError(1, 3, 0)
	Local $pResumeThread = _HUNGAPP_GetAddress($hKernel32, "ResumeThread")
	If @error Then Return SetError(1, 4, 0)
	Local $pSwitchToThread = _HUNGAPP_GetAddress($hKernel32, "SwitchToThread")
	If @error Then Return SetError(1, 5, 0)
	Local $pExitProcess = _HUNGAPP_GetAddress($hKernel32, "ExitProcess")
	If @error Then Return SetError(1, 6, 0)
	Local $pGetLastError = _HUNGAPP_GetAddress($hKernel32, "GetLastError")
	If @error Then Return SetError(1, 7, 0)
	; Get handle of the main thread
	Local $hProcessPseudo = _HUNGAPP_GetCurrentProcess()
	Local $hThreadPseudo = _HUNGAPP_GetCurrentThread()
	; Pseude to real handle:
	Local $hMainThraed = _HUNGAPP_DuplicateHandle($hProcessPseudo, $hThreadPseudo, $hProcessPseudo, 0, 0, 2) ; DUPLICATE_SAME_ACCESS
	If @error Then Return SetError(2, 0, 0)
	; Allocate memory space
	Local $iCodeSize = 512 ; for example. It's more than enough
	Local $pCodeBuffer = _HUNGAPP_VirtualAlloc(0, $iCodeSize)
	If @error Then
		_HUNGAPP_CloseHandle($hMainThraed) ; close opened handle
		Return SetError(3, 0, 0)
	EndIf
	; AutoIt function to call from another thread
	Local $hFunction = DllCallbackRegister($sFunction, "int", "lparam;wparam")
	If @error Then
		_HUNGAPP_CloseHandle($hMainThraed) ; close opened handle
		Return SetError(4, 0, 0)
	EndIf
	; Structure in place of allocated space
	Local $tCodeBuffer = DllStructCreate("byte[" & $iCodeSize & "]", $pCodeBuffer)
	; Define ERROR_TIMEOUT value
	Local $iERROR_TIMEOUT = 1460
	If @OSVersion = "WIN_2000" Then $iERROR_TIMEOUT = 0
	; Write code
	If @AutoItX64 Then
		DllStructSetData($tCodeBuffer, 1, _
				"0x" & _
				"4883EC" & _HUNGAPP_SwapEndian(72, 1) & _                            ; sub rsp, 72d ;<- expand the stack
				"" & _ ; 10. SKIP SLEEP  (2 bytes code)
				"EB" & _HUNGAPP_SwapEndian(17, 1) & _                                ; jmp 17d
				"" & _ ; 20. SLEEP FOR $iInterval  (17 bytes code)
				"B9" & _HUNGAPP_SwapEndian($iInterval, 4) & _                        ; mov ecx, $iInterval
				"48B8" & _HUNGAPP_SwapEndian($pSleep, 8) & _                         ; mov rax, $pSleep
				"FFD0" & _                                                           ; call rax
				"" & _ ; 30. CHECK IF BLOCKED  (63 bytes code)
				"C7442428" & _HUNGAPP_SwapEndian($iHungAppTime, 4) & _               ; mov [rsp+40], $iHungAppTime
				"C7442420" & _HUNGAPP_SwapEndian(0, 4) & _                           ; mov [rsp+32], 0 (SMTO_NORMAL)
				"49B9" & _HUNGAPP_SwapEndian(0, 8) & _                               ; mov r9, 0
				"49B8" & _HUNGAPP_SwapEndian(0, 8) & _                               ; mov r8, 0
				"BA" & _HUNGAPP_SwapEndian(0, 4) & _                                 ; mov edx, WM_NULL
				"48B9" & _HUNGAPP_SwapEndian($hWnd, 8) & _                           ; mov rcx,  $hWnd ;<- window handle
				"48B8" & _HUNGAPP_SwapEndian($pSendMessageTimeout, 8) & _            ; mov rax, $pSendMessageTimeout
				"FFD0" & _                                                           ; call rax
				"" & _ ; 40. SEE WHAT'S RETURNED  (3 bytes code)
				"4885C0" & _                                                         ; test rax, rax
				"" & _ ; 50. CONDITION - IF 'TRUE' RETURNED THEN JUMP TO 20  (2 bytes code)
				"75" & _HUNGAPP_SwapEndian(-85, 1) & _                               ; jne -85d ;<- if SendMessageTimeout returned TRUE jump back 85 bytes
				"" & _ ; 60. CHECK LastError (12 bytes code)
				"48B8" & _HUNGAPP_SwapEndian($pGetLastError, 8) & _                  ; mov rax, $pGetLastError
				"FFD0" & _                                                           ; call rax
				"" & _ ; 70. COMPARE RETURNED WITH ERROR_TIMEOUT VALUE (5 bytes code)
				"3D" & _HUNGAPP_SwapEndian($iERROR_TIMEOUT, 4) & _                   ; cmp eax, $iERROR_TIMEOUT
				"" & _ ; 80. IF NOT $iERROR_TIMEOUT THEN JUMP TO 20 (2bytes code)
				"75" & _HUNGAPP_SwapEndian(-104, 1) & _                              ; jne -104d
				"" & _ ; 90. INCREASE SUSPEND COUNT FOR THE MAIN THREAD  (22 bytes code)
				"48B9" & _HUNGAPP_SwapEndian($hMainThraed, 8) & _                    ; mov rcx, $hMainThraed
				"48B8" & _HUNGAPP_SwapEndian($pSuspendThread, 8) & _                 ; mov rax, $pSuspendThread
				"FFD0" & _                                                           ; call rax
				"" & _ ; 100. CALL AUTOIT FUNCTION  (32 bytes code)
				"48BA" & _HUNGAPP_SwapEndian($vParam2, 8) & _                        ; mov rdx, $vParam2
				"48B9" & _HUNGAPP_SwapEndian($vParam1, 8) & _                        ; mov rcx, $vParam1
				"48B8" & _HUNGAPP_SwapEndian(DllCallbackGetPtr($hFunction), 8) & _   ; mov rax, $pFunction
				"FFD0" & _                                                           ; call rax
				"" & _ ; 110. SEE IF $HUNGAPP_CONTINUEHANDLER IS RETURNED FROM THE CALL  (5 bytes code)
				"3D" & _HUNGAPP_SwapEndian(259, 4) & _                               ; cmp eax, 259d ;<- compare returned with 259
				"" & _ ; 120. CONDITION - IF NOT THEN JUMP TO 160  (2 bytes code)
				"75" & _HUNGAPP_SwapEndian(39, 1) & _                                ; jne 39d ;<- if not 259 returned jump forward 39 bytes
				"" & _ ; 130. IF YES THEN CALL ResumeThread FUNCTION  (22 bytes code)
				"48B9" & _HUNGAPP_SwapEndian($hMainThraed, 8) & _                    ; mov rcx, $hMainThraed
				"48B8" & _HUNGAPP_SwapEndian($pResumeThread, 8) & _                  ; mov rax, $pResumeThread
				"FFD0" & _                                                           ; call rax
				"" & _ ; 140. CALL SwitchToThread FUNCTION  (12 byes code)
				"48B8" & _HUNGAPP_SwapEndian($pSwitchToThread, 8) & _                ; mov rax, $pSwitchToThread
				"FFD0" & _                                                           ; call rax
				"" & _ ; 150. JUMP BACK TO 20  (5 bytes code)
				"E9" & _HUNGAPP_SwapEndian(-187, 4) & _                              ; jmp -187d ; jump back 187 bytes
				"" & _ ; 160. SEE WHAT'S RETURNED. THIS TIME TRUE/FALSE  (2 bytes code)
				"85C0" & _                                                           ; test eax, eax
				"" & _ ; 170. CONDITION - IF FALSE THEN JUMP TO 190  (2 bytes code)
				"74" & _HUNGAPP_SwapEndian(17, 1) & _                                ; je 17d ;<- if 0 returned jump forward 17 bytes
				"" & _ ; 180. CONDITION - IF TRUE THEN CALL ExitProcess FUNCTION  (17 bytes code)
				"B9" & _HUNGAPP_SwapEndian($iExitCode, 4) & _                        ; mov ecx, $iExitCode
				"48B8" & _HUNGAPP_SwapEndian($pExitProcess, 8) & _                   ; mov rax, $pExitProcess
				"FFD0" & _                                                           ; call rax
				"" & _ ; 190. RESUME MAIN THREAD  (22 bytes code)
				"48B9" & _HUNGAPP_SwapEndian($hMainThraed, 8) & _                    ; mov rcx, $hMainThraed
				"48B8" & _HUNGAPP_SwapEndian($pResumeThread, 8) & _                  ; mov rax, $pResumeThread
				"FFD0" & _                                                           ; call rax
				"" & _ ; 200. SET RET VALUE TO 0
				"4831C0" & _                                                         ; xor rax, rax
				"4883C4" & _HUNGAPP_SwapEndian(72, 1) & _                            ; add rsp, 72d ;<- shrink stack
				"" & _ ; 210. RETURN
				"C3" _                                                               ; ret
				)
	Else
		DllStructSetData($tCodeBuffer, 1, _
				"0x" & _
				"" & _ ; 10. SKIP SLEEP  (2 bytes code)
				"EB" & _HUNGAPP_SwapEndian(12, 1) & _                                ; jmp 12d
				"" & _ ; 20. SLEEP FOR $iInterval  (12 bytes code)
				"68" & _HUNGAPP_SwapEndian($iInterval, 4) & _                        ; push $iInterval
				"B8" & _HUNGAPP_SwapEndian($pSleep, 4) & _                           ; mov eax, $pSleep
				"FFD0" & _                                                           ; call eax
				"" & _ ; 30. CHECK IF BLOCKED  (38 bytes code)
				"53" & _                                                             ; push ebx
				"68" & _HUNGAPP_SwapEndian($iHungAppTime, 4) & _                     ; push $iHungAppTime
				"68" & _HUNGAPP_SwapEndian(0, 4) & _                                 ; push 0 (SMTO_NORMAL)
				"68" & _HUNGAPP_SwapEndian(0, 4) & _                                 ; push 0
				"68" & _HUNGAPP_SwapEndian(0, 4) & _                                 ; push 0
				"68" & _HUNGAPP_SwapEndian(0, 4) & _                                 ; push WM_NULL
				"68" & _HUNGAPP_SwapEndian($hWnd, 4) & _                             ; push $hWnd
				"B8" & _HUNGAPP_SwapEndian($pSendMessageTimeout, 4) & _              ; mov eax, $pSendMessageTimeout
				"FFD0" & _                                                           ; call eax
				"" & _ ; 40. SEE WHAT'S RETURNED  (2 bytes code)
				"85C0" & _                                                           ; test eax, eax
				"" & _ ; 50. CONDITION - IF 'TRUE' RETURNED THEN JUMP TO 20  (2 bytes code)
				"75" & _HUNGAPP_SwapEndian(-54, 1) & _                               ; jne -54d ;<- if SendMessageTimeout returned TRUE jump back 54 bytes
				"" & _ ; 60. CHECK LastError (7 bytes code)
				"B8" & _HUNGAPP_SwapEndian($pGetLastError, 4) & _                    ; mov eax, $pGetLastError
				"FFD0" & _                                                           ; call eax
				"" & _ ; 70. COMPARE RETURNED WITH ERROR_TIMEOUT VALUE (5 bytes code)
				"3D" & _HUNGAPP_SwapEndian($iERROR_TIMEOUT, 4) & _                   ; cmp eax, $iERROR_TIMEOUT
				"" & _ ; 80. IF NOT $iERROR_TIMEOUT THEN JUMP TO 20 (2bytes code)
				"75" & _HUNGAPP_SwapEndian(-68, 1) & _                               ; jmp -68
				"" & _ ; 90. INCREASE SUSPEND COUNT FOR THE MAIN THREAD  (12 bytes code)
				"68" & _HUNGAPP_SwapEndian($hMainThraed, 4) & _                      ; push $hMainThraed
				"B8" & _HUNGAPP_SwapEndian($pSuspendThread, 4) & _                   ; mov eax,  $pSuspendThread
				"FFD0" & _                                                           ; call eax
				"" & _ ; 100. CALL AUTOIT FUNCTION  (17 bytes code)
				"68" & _HUNGAPP_SwapEndian($vParam2, 4) & _                          ; push $vParam2
				"68" & _HUNGAPP_SwapEndian($vParam1, 4) & _                          ; push $vParam1
				"B8" & _HUNGAPP_SwapEndian(DllCallbackGetPtr($hFunction), 4) & _     ; mov eax, $pFunction
				"FFD0" & _                                                           ; call eax
				"" & _ ; 110. SEE IF $HUNGAPP_CONTINUEHANDLER IS RETURNED FROM THE CALL  (5 bytes code)
				"3D" & _HUNGAPP_SwapEndian(259, 4) & _                               ; cmp eax, 259d ;<- compare returned with 259
				"" & _ ; 120. CONDITION - IF NOT THEN JUMP TO 160  (2 bytes code)
				"75" & _HUNGAPP_SwapEndian(21, 1) & _                                ; jne 21d ;<- if not 259 returned jump forward 21 bytes
				"" & _ ; 130. IF YES THEN CALL ResumeThread FUNCTION  (12 bytes code)
				"68" & _HUNGAPP_SwapEndian($hMainThraed, 4) & _                      ; push  $hMainThraed
				"B8" & _HUNGAPP_SwapEndian($pResumeThread, 4) & _                    ; mov eax, $pResumeThread
				"FFD0" & _                                                           ; call eax
				"" & _ ; 140. CALL SwitchToThread FUNCTION  (7 byes code)
				"B8" & _HUNGAPP_SwapEndian($pSwitchToThread, 4) & _                  ; mov eax, $pSwitchToThread
				"FFD0" & _                                                           ; call eax
				"" & _ ; 150. JUMP BACK TO 30  (2 bytes code)
				"EB" & _HUNGAPP_SwapEndian(-113, 1) & _                              ; jmp -113d ;<- jump back 113 bytes
				"" & _ ; 160. SEE WHAT'S RETURNED. THIS TIME TRUE/FALSE  (2 bytes code)
				"85C0" & _                                                           ; test eax, eax
				"" & _ ; 170. CONDITION - IF FALSE THEN JUMP TO 190  (2bytes code)
				"74" & _HUNGAPP_SwapEndian(12, 1) & _                                ; je 12d ;<- if 0 returned jump forward 12 bytes
				"" & _ ; 180. CONDITION - IF TRUE THEN CALL ExitProcess FUNCTION  (12 bytes code)
				"68" & _HUNGAPP_SwapEndian($iExitCode, 4) & _                        ; push $iExitCode
				"B8" & _HUNGAPP_SwapEndian($pExitProcess, 4) & _                     ; mov eax, $pExitProcess
				"FFD0" & _                                                           ; call eax
				"" & _ ; 190. RESUME MAIN THREAD  (12 bytes code)
				"68" & _HUNGAPP_SwapEndian($hMainThraed, 4) & _                      ; push $hMainThraed
				"B8" & _HUNGAPP_SwapEndian($pResumeThread, 4) & _                    ; mov eax, $pResumeThread
				"FFD0" & _                                                           ; call eax
				"" & _ ; 130. SET RET VALUE TO 0
				"31C0" & _                                                           ; xor eax, eax
				"" & _ ; 140. RETURN
				"C3" _                                                               ; ret
				)
	EndIf
	; Set PAGE_EXECUTE for the code buffer
	_HUNGAPP_VirtualProtect($pCodeBuffer, $iCodeSize, 16) ; PAGE_EXECUTE
	If @error Then
		_HUNGAPP_CloseHandle($hMainThraed) ; close opened handle
		_HUNGAPP_VirtualFree($pCodeBuffer) ; free allocated memory
		Return SetError(5, 0, 0)
	EndIf
	; Create hread to run code in
	Local $hThread = _HUNGAPP_CreateThread($pCodeBuffer)
	If @error Then
		_HUNGAPP_CloseHandle($hMainThraed) ; close opened handle
		_HUNGAPP_VirtualFree($pCodeBuffer) ; free allocated memory
		Return SetError(6, 0, 0)
	EndIf
	; Arrange return
	Local $aOut[5] = [0x48756E67, $hMainThraed, $pCodeBuffer, $hThread, $hFunction] ; first element is magic num
	; Return sucess
	Return $aOut
EndFunc   ;==>_OnHungAppRegisterEx

; #FUNCTION# ;===============================================================================
; Name...........: _OnHungAppUnRegister
; Description ...: Unregisters function registerd with _OnHungAppRegister and frees all resources
; Syntax.........: _OnHungAppUnRegister(ByRef $hHandler)
; Parameters ....: $hHandler - Specific handle returned by _OnHungAppRegister or _OnHungAppRegisterEx functions.
; Return values .: Success - Returns 1
;                  Failure - Returns 0 and sets @error:
;                  |1 - Invalid handle.
; Author ........: trancexx
;============================================================================================
Func _OnHungAppUnRegister(ByRef $hHandler)
	Local $aData = $hHandler
	; Check if the 'handle' is valid
	If UBound($aData) <> 5 Or $aData[0] <> 0x48756E67 Then Return SetError(1, 0, 0)
	; Collect data
	Local $hMainThraed = $aData[1]
	Local $pCodeBuffer = $aData[2]
	Local $hThread = $aData[3]
	Local $hFunction = $aData[4]
	; Free everything
	If $hMainThraed Then _HUNGAPP_CloseHandle($hMainThraed) ; close opened handle
	If $hThread Then
		_HUNGAPP_TerminateThread($hThread) ; kill the thread
		_HUNGAPP_CloseHandle($hThread) ; close opened handle
	EndIf
	If $pCodeBuffer Then _HUNGAPP_VirtualFree($pCodeBuffer) ; free allocated memory
	If $hFunction Then DllCallbackFree($hFunction)
	$hHandler = 0
	; Return success
	Return 1
EndFunc   ;==>_OnHungAppUnRegister

; #INTERNAL FUNCTIONS DOWN BELOW# ;==========================================================
Func _HUNGAPP_SwapEndian($iValue, $iSize = 0)
	If $iSize Then
		Local $sPadd = "00000000"
		Return Hex(BinaryMid($iValue, 1, $iSize)) & StringLeft($sPadd, 2 * ($iSize - BinaryLen($iValue)))
	EndIf
	Return Hex(Binary($iValue))
EndFunc   ;==>_HUNGAPP_SwapEndian

Func _HUNGAPP_GetModuleHandle($vModule = 0)
	Local $sType = "wstr"
	If Not $vModule Then $sType = "ptr"
	Local $aCall = DllCall($hHUNGAPPDLL__KERNEL32, "ptr", "GetModuleHandleW", $sType, $vModule)
	If @error Or Not $aCall[0] Then Return SetError(1, 0, 0)
	Return $aCall[0]
EndFunc   ;==>_HUNGAPP_GetModuleHandle

Func _HUNGAPP_GetAddress($hModule, $vFuncName)
	Local $sType = "str"
	If IsNumber($vFuncName) Then $sType = "int" ; if ordinal value passed
	Local $aCall = DllCall($hHUNGAPPDLL__KERNEL32, "ptr", "GetProcAddress", "handle", $hModule, $sType, $vFuncName)
	If @error Or Not $aCall[0] Then Return SetError(1, 0, 0)
	Return $aCall[0]
EndFunc   ;==>_HUNGAPP_GetAddress

Func _HUNGAPP_GetCurrentProcess()
	Local $aCall = DllCall($hHUNGAPPDLL__KERNEL32, "handle", "GetCurrentProcess")
	If @error Or Not $aCall[0] Then Return SetError(1, 0, 0)
	Return $aCall[0]
EndFunc   ;==>_HUNGAPP_GetCurrentProcess

Func _HUNGAPP_GetCurrentThread()
	Local $aCall = DllCall($hHUNGAPPDLL__KERNEL32, "handle", "GetCurrentThread")
	If @error Or Not $aCall[0] Then Return SetError(1, 0, 0)
	Return $aCall[0]
EndFunc   ;==>_HUNGAPP_GetCurrentThread

Func _HUNGAPP_DuplicateHandle($hSourceProcessHandle, $hSourceHandle, $hTargetProcessHandle, $iDesiredAccess, $fInheritHandle, $iOptions)
	Local $aCall = DllCall($hHUNGAPPDLL__KERNEL32, "bool", "DuplicateHandle", _
			"handle", $hSourceProcessHandle, _
			"handle", $hSourceHandle, _
			"handle", $hTargetProcessHandle, _
			"handle*", 0, _
			"dword", $iDesiredAccess, _
			"bool", $fInheritHandle, _
			"dword", $iOptions)
	If @error Or Not $aCall[0] Then Return SetError(1, 0, 0)
	Return $aCall[4]
EndFunc   ;==>_HUNGAPP_DuplicateHandle

Func _HUNGAPP_CloseHandle($hHandle)
	Local $aCall = DllCall($hHUNGAPPDLL__KERNEL32, "bool", "CloseHandle", "handle", $hHandle)
	If @error Or Not $aCall[0] Then Return SetError(1, 0, 0)
	Return 1
EndFunc   ;==>_HUNGAPP_CloseHandle

Func _HUNGAPP_VirtualAlloc($pAddress, $iSize, $iAllocationType = 0x1000, $iProtect = 4) ; default is MEM_COMMIT and PAGE_READWRITE
	Local $aCall = DllCall($hHUNGAPPDLL__KERNEL32, "ptr", "VirtualAlloc", "ptr", $pAddress, "dword_ptr", $iSize, "dword", $iAllocationType, "dword", $iProtect)
	If @error Or Not $aCall[0] Then Return SetError(1, 0, 0)
	Return $aCall[0]
EndFunc   ;==>_HUNGAPP_VirtualAlloc

Func _HUNGAPP_VirtualFree($pAddress, $iSize = 0, $iFreeType = 0x8000) ; MEM_RELEASE default
	Local $aCall = DllCall($hHUNGAPPDLL__KERNEL32, "bool", "VirtualFree", "ptr", $pAddress, "dword_ptr", $iSize, "dword", $iFreeType)
	If @error Or Not $aCall[0] Then Return SetError(1, 0, 0)
	Return $aCall[0]
EndFunc   ;==>_HUNGAPP_VirtualFree

Func _HUNGAPP_VirtualProtect($pAddress, $iSize, $iProtection)
	Local $aCall = DllCall($hHUNGAPPDLL__KERNEL32, "bool", "VirtualProtect", "ptr", $pAddress, "dword_ptr", $iSize, "dword", $iProtection, "dword*", 0)
	If @error Or Not $aCall[0] Then Return SetError(1, 0, 0)
	Return 1
EndFunc   ;==>_HUNGAPP_VirtualProtect

Func _HUNGAPP_CreateThread($pAddress)
	Local $aCall = DllCall($hHUNGAPPDLL__KERNEL32, "handle", "CreateThread", _
			"ptr", 0, _
			"dword_ptr", 0, _
			"ptr", $pAddress, _
			"ptr", 0, _
			"dword", 0, _
			"dword*", 0)
	If @error Or Not $aCall[0] Then Return SetError(1, 0, 0)
	Return $aCall[0]
EndFunc   ;==>_HUNGAPP_CreateThread

Func _HUNGAPP_SuspendThread($hTread)
	Local $aCall = DllCall($hHUNGAPPDLL__KERNEL32, "dword", "SuspendThread", "handle", $hTread)
	If @error Or $aCall[0] = -1 Then Return SetError(1, 0, -1)
	Return $aCall[0]
EndFunc   ;==>_HUNGAPP_SuspendThread

Func _HUNGAPP_ResumeThread($hTread)
	Local $aCall = DllCall($hHUNGAPPDLL__KERNEL32, "dword", "ResumeThread", "handle", $hTread)
	If @error Or $aCall[0] = -1 Then Return SetError(1, 0, -1)
	Return $aCall[0]
EndFunc   ;==>_HUNGAPP_ResumeThread

Func _HUNGAPP_TerminateThread($hTread, $iExitCode = 0)
	Local $aCall = DllCall($hHUNGAPPDLL__KERNEL32, "dword", "TerminateThread", "handle", $hTread, "dword", $iExitCode)
	If @error Or Not $aCall[0] Then Return SetError(1, 0, 0)
	Return $aCall[0]
EndFunc   ;==>_HUNGAPP_TerminateThread

Func _HUNGAPP_IsHungAppWindow($hWnd)
	Local $aCall = DllCall("user32.dll", "bool", "IsHungAppWindow", "hwnd", $hWnd)
	If @error Then Return SetError(1, 0, 0)
	Return $aCall[0]
EndFunc
;============================================================================================